@@ -30,27 +30,29 @@ import numpy as np
 from laygo2.object.physical import *
 
 
-class RoutingChannel:
+class RoutingMesh:
     grid = None
-    """laygo2.object.grid.RoutingGrid: A routing grid object that this routing channel refers to.
-    """
-
-    direction = "horizontal"
-    """str: 
+    """laygo2.object.grid.RoutingGrid: A routing grid object that this routing mesh refers to.
     """
 
     tracks = dict()
-    """dict: A dictionary that contains net names as keys and routing track index as values.
+    """dict: A dictionary that contains net names as keys and horizontal routing track index as values.
     """
 
     nodes = []
     """list: A list that contains coordinates and layout object to connected through the routing channel.
     """
 
-    range = [None, None]
+    hrange = [None, None]
     """[int or None, int or None]: The minimum and maximum track indices covered by the 
-    routing channel, for automatic routing.
+    horizontal routing channel, for automatic routing.
     """
+    
+    vrange = [None, None]
+    """[int or None, int or None]: The minimum and maximum track indices covered by the 
+    vertical routing channel, for automatic routing.
+    """
+
 
     def __init__(self, grid, tracks:dict = None, nodes:list = None):
         """Constructor"""
@@ -61,8 +63,10 @@ class RoutingChannel:
         if nodes is not None:
             self.nodes = nodes
 
-    def add_track(self, netname:str, index:int):
-        self.tracks[netname] = index
+    def add_track(self, name:str, index:int, netname:str = None):
+        if netname is None:
+            netname = name
+        self.tracks[name] = [index, netname]
     
     def add_node(self, obj):
         if isinstance(obj, list):
@@ -70,36 +74,53 @@ class RoutingChannel:
         else:
             self.nodes.append(obj)
 
-    def generate(self, tracks:dict = None, nodes:list = None):
-        if tracks is None:
-            tracks = self.tracks
-        if nodes is None:
-            nodes = self.nodes
-
+    def generate(self):
         g = self.grid
-        r_list = []
+        tr = self.tracks
+        nds = self.nodes
 
-        for tn, ti in tracks.items():  # for each track
-            # Find out routing points.
+        return self._construct_rdict(grid=g, tracks=tr, nodes=nds) 
+
+    def _construct_rdict(self, grid, tracks, nodes):
+        """Construct a routing directiony for each direction for generate()."""
+        
+        # Variables for VirtualInstance construction.
+        nelements = dict()
+        pins = dict()
+        for tn, t in tracks.items():  # for each track
+            ti = t[0]  # track index
+            tnn = t[1]  # track netname
             mn = []
             for n in nodes:
                 if isinstance(n, Instance) or isinstance(n, VirtualInstance):
                     for pn, p in n.pins.items():
-                        if p.netname == tn:
-                            mn.append(g.mn(p.xy)[0])
+                        if p.netname == tnn:
+                            mn.append(grid.mn(p)[0])
                 elif isinstance(n, Rect):
-                    if n.netname == tn:
-                        mn.append(g.mn(n.xy)[0])
+                    if n.netname == tnn:
+                        mn.append(grid.mn(n)[0])
             # Do routing
-            if self.direction == "horizontal":
-                t = [None, ti]
+            if ti[0] is None:  # horizontal track
+                t = [None, ti[1]]
             else:
-                t = [ti, None]
-            r = g.route_via_track(mn=mn, track=t)
-            r_list.append(r[-1])
-        return r_list
+                t = [ti[0], None]
+            r = grid.route_via_track(mn=mn, track=t)
 
-
-# class RoutingMesh:
-#    # TODO: implement this
-#    pass
+            # Wrap the generated routing structure into a VirtualInstance
+            for i, _r in enumerate(r):
+                if isinstance(_r, list):
+                    for j, __r in enumerate(_r):
+                        nelements[tn+"_"+str(i)+"_"+str(j)]=__r
+                else:
+                    nelements[tn+"_"+str(i)]=_r
+            pins[tn] = Pin(xy=r[-1].xy, layer=r[-1].layer, netname=tnn)
+        rinst = VirtualInstance(
+            name=tn,
+            xy=np.array([0, 0]),
+            libname="mylib",
+            cellname="rtrack_" + tn + "_" + tnn,
+            native_elements=nelements,
+            transform="R0",
+            pins=pins,
+        )
+        return rinst
